﻿using UnityEngine;
using System.Collections;

/// <summary>
/// Author: Adam Waggoner
/// Deals with placing and removing blocks, and the time associated with that
/// </summary>

public class PlayerController : MonoBehaviour {

    /// <summary>
    /// Layermask to raycast with (should only contain the "Tile" layer)
    /// </summary>
    [SerializeField]
    private LayerMask _TileLayer;

    /// <summary>
    /// Cursor prefab to show player's selection
    /// </summary>
    [SerializeField]
    private GameObject _Cursor;

    /// <summary>
    /// The trench to replace sand with
    /// </summary>
    [SerializeField]
    private GameObject _TrenchPrefab, _WallPrefab;

    /// <summary>
    /// Currently selected object
    /// </summary>
    private GameObject _SelectedObject;

    /// <summary>
    /// The tile component connected to the selected object
    /// </summary>
    private Tile _SelectedTile;

    /// <summary>
    /// The actual instance of the cursor
    /// </summary>
    private GameObject _CursorInstance;

    /// <summary>
    /// Reference to the map
    /// </summary>
    private GridManager _TileGrid;

    /// <summary>
    /// How long right click has been held
    /// </summary>
    private float _LeftClickTimer, _RightClickTimer;

    /// <summary>
    /// Reference to the resourcemanager
    /// </summary>
    private ResourceManager _Resources;

    /// <summary>
    /// Temporary representation of the object being built
    /// </summary>
    private GameObject _TempBuildObject;

    /// <summary>
    /// Sounds for placement and digging
    /// </summary>
    [SerializeField]
    private AudioClip _PlaceSound, _DigSound;

    private AudioSource _SoundSource;

    public GameObject _DigParticles;
    private GameObject _ParticleInstance;

    //Does the player need to click again before they can dig
    private bool _Cooldown;

    // Use this for initialization
    void Start () {
        _CursorInstance = GameObject.Instantiate(_Cursor,_Cursor.transform.position,Quaternion.identity) as GameObject;
        _CursorInstance.SetActive(false);

        _Resources = FindObjectOfType<ResourceManager>();

        _TileGrid = FindObjectOfType<GridManager>();

        _SoundSource = FindObjectOfType<AudioSource>();
	}
	
	// Update is called once per frame
	void Update () {
        _FindSelectedTile();

        if (Input.GetMouseButton(0) && _RightClickTimer == 0 && !_Cooldown)
        { 

            if (_SelectedTile != null && _SelectedTile.removeTime != 0)
            {
                _UpdateLeftClick();
                if (_LeftClickTimer == 0)
                    _SoundSource.PlayOneShot(_DigSound);
                if (Input.GetMouseButtonDown(0))
                { _ParticleInstance = Instantiate(_DigParticles, new Vector3(_SelectedTile.GetCol(), 0, _SelectedTile.GetRow()) + _DigParticles.transform.position, _DigParticles.transform.rotation) as GameObject; }
            }
            _LeftClickTimer += Time.deltaTime;


        }

        else if (Input.GetMouseButton(1) && _LeftClickTimer == 0)
        {

            if (_SelectedTile != null && _SelectedTile.canBuild && _Resources.GetSand() > 0)
            {
                if (_RightClickTimer == 0)
                    _SoundSource.PlayOneShot(_PlaceSound);
                _UpdateRightClick();
            }

            _RightClickTimer += Time.deltaTime;
        }
        else if (Input.GetMouseButtonUp(0))
        {
            _EndParticles();
            _Cooldown = false;
            _Deselect();
        }
        else if (Input.GetMouseButtonUp(1))
        {
            _Deselect();
        }

    }

    private void _UpdateLeftClick()
    {
        float completion = 1 - _LeftClickTimer / _SelectedTile.removeTime;

        _SetMaterialAlpha(completion, _SelectedObject);

        if (_LeftClickTimer >= _SelectedTile.removeTime)
        {
            _Cooldown = true;
            _EndParticles();
            _Resources.GainSand(1);
            _TileGrid.ReplaceTileOffset(_TrenchPrefab, _SelectedTile.GetCol(), _SelectedTile.GetRow(), _TrenchPrefab.transform.position);
            
        }
    }

    private void _UpdateRightClick()
    {
        GameObject pref = _SelectedTile.UpgradesTo;
        float buildTime = pref.GetComponent<Tile>().placementTime;
        float completion = _RightClickTimer / buildTime;

        if (_TempBuildObject == null)
        {
            _TempBuildObject = Instantiate(pref, new Vector3(_SelectedTile.GetCol(),0,_SelectedTile.GetRow()) + pref.GetComponent<Tile>().transform.position, pref.GetComponent<Tile>().transform.rotation) as GameObject;
            Destroy(_TempBuildObject.GetComponent<Rigidbody>());
            Destroy(_TempBuildObject.GetComponent<Collider>());
        }

        _SetMaterialAlpha(completion, _TempBuildObject);

        if (_RightClickTimer >= buildTime)
        {
            _Resources.LoseSand(1);
            _TileGrid.ReplaceTileOffset(pref, _SelectedTile.GetCol(), _SelectedTile.GetRow(), pref.transform.position);
            _RightClickTimer = 0;
        }
    }

    /// <summary>
    /// Cast a ray from the mouse position to determine which object is selected
    /// </summary>
    void _FindSelectedTile()
    {
        Ray mouseRay = Camera.main.ScreenPointToRay(Input.mousePosition);
        RaycastHit hit;
        Physics.Raycast(mouseRay, out hit,100,_TileLayer);

        if (hit.rigidbody != null)
        {
            if (hit.rigidbody.gameObject == _SelectedObject)
                return;
            else
            {
                if (_CursorInstance != null)
                {
                    _Deselect();
                }

                _Select(hit.rigidbody.gameObject);
            }
        }
        else
        {
            _Deselect();
        }

    }

    private void _SetMaterialAlpha(float val, GameObject obj)
    {
        if (_SelectedObject != null)
        {
            Renderer tmpRend = obj.GetComponent<Renderer>();
            for (int i = 0; i < tmpRend.materials.Length; i++)
            {
                Color tmpColor = tmpRend.materials[i].color;
                tmpColor.a = val;
                tmpRend.materials[i].color = tmpColor;
            }
        }
    }

    /// <summary>
    /// Change the selected object to "selectObj"
    /// </summary>
    /// <param name="selectObj"></param>
    private void _Select(GameObject selectObj)
    {
        _SelectedObject = selectObj;
        _CursorInstance = Instantiate(_Cursor, _Cursor.transform.position, Quaternion.identity) as GameObject;
        _SelectedTile = _SelectedObject.GetComponent<Tile>();
        _CursorInstance.transform.position += new Vector3(_SelectedTile.GetCol(), 0, _SelectedTile.GetRow());

        if (_ParticleInstance != null && !_ParticleInstance.GetComponent<ParticleSystem>().isPaused && !_Cooldown)
        {
            _ParticleInstance.transform.position = new Vector3(_SelectedTile.GetCol(), _ParticleInstance.transform.position.y, _SelectedTile.GetRow());
        }
    }

    private void _EndParticles()
    {
        if (_ParticleInstance == null)
            return;

        _ParticleInstance.GetComponent<ParticleSystem>().Stop();
        //_ParticleInstance.GetComponent<ParticleSystem>().isPlaying = false;
        Destroy(_ParticleInstance,1);
    }

    private void _Deselect()
    {
        _RightClickTimer = 0;
        _LeftClickTimer = 0;

        if (_SelectedObject != null)
        _SetMaterialAlpha(1, _SelectedObject);

        if (_TempBuildObject != null)
        {
            Destroy(_TempBuildObject);
        }

        Destroy(_CursorInstance);
    }
}
